

#include "gamegine.h"
#include "sfx.h"
#include "cam_ai.h"
#include "brain.h"
#include "vpixcon.h"

#define MAX_LENS_CORONA 9

vfx_lens_corona lens_glare;
vfx_lens_corona *lens_flare = NULL;
double vfx_timer = 0;


/* **************************************************************
************************************************************** */
void vfx_dest() {

   if (lens_flare) {
      delete [] lens_flare;
      lens_flare = NULL;
   }
   
}


/* **************************************************************
************************************************************** */
int vfx_lens_flare(void *buffer, quark *sol, gameatom *solsys) {

   vector2f axis;
   float tf;
   double dot, t;
   vector4f pos, pos2, center;
   vector3f normal;
   int i;
   gameatom *atr, *ctr;
   quark *obstacle;

   if (current_camera->query_frustum_clip(sol->old_state.bradius, sol->old_state.center, &tf, ~FRUSTUM_CLIP_BACK))
      return 0;

   subeqarray3(pos, sol->old_state.center, current_camera->location);
   pos[3] = 1.0f;

   // dont do los calc w/ atom if camera is inside
   for (ctr=NULL, atr=pixcon_object_list; atr; atr=(gameatom *)atr->next) {
      if (!atr->query_whatwasi(OBJECT_CAMERA_CONTROL))
         continue;

      if (((camera_control *)atr)->query_view_type() != VIEW_CHASE)
         ctr = ((camera_control *)atr)->query_target()->object;

      break;
   }

   // put light in screen space
   copyarray3(center, sol->old_state.center);
   center[3] = 1;
   matvecmulto(current_camera->transform, center);
   current_camera->map2screen(center);

   // calc axis (X) of the lens flare
   axis[0] = WINX*0.5f - center[0];
   axis[1] = WINY*0.5f - center[1];

   t = 1.0 - 2.0*SQRT( dotproduct2(axis, axis)/(WINX*WINX + WINY*WINY));

   if (t > 0) {
      dot = t*t;
      t = 0.1 + 0.9 * t;
      dot *= t;
   }
   
   else {
      dot = 0;
      t = 0.1f;
   }

   // glare   
   lens_glare.render((mapul *)buffer, (int)vfx_timer, center, axis, (float)dot);
   
   // line intersect all for something in the way....
   for (atr=pixcon_object_list; atr; atr=(gameatom *)atr->next)
      if (atr != ctr && atr != solsys && !query_clear_los(NULL, atr, NULL, current_camera->location, pos, &obstacle, pos2, normal, COL_POLYGON))
         return 0;

   // flare
   for (i = 0; i < MAX_LENS_CORONA; i++)
      lens_flare[i].render((mapul *)buffer, (int)vfx_timer, center, axis, (float)t);

   return 1;
}


/* **************************************************************
************************************************************** */
void vfx_init() {

   vector4uc white = { 255, 255, 255, 0 };

   vfx_timer = 0;
   
   // read in lens flare vf/x

   if (lens_flare)
      delete [] lens_flare;

   lens_flare = new vfx_lens_corona[MAX_LENS_CORONA];

/*
   vector4uc red = { 255, 0, 0, 0 };
   vector4uc violet = { 45, 0, 255, 0 };
   lens_flare[0].init("textures/flare3.bmp", 0.2f, 0.04f*WINX, red, 1.0f, 0.6f);
   lens_flare[1].init("textures/flare4.bmp", 0.4f, 0.1f*WINX, red, 1.0f, 0.4f);
   lens_flare[2].init("textures/flare2.bmp", 0.6f, 0.2f*WINX, red, 1.0f, 0.3f);
   lens_flare[3].init("textures/flare5.bmp", 0.8f, 0.05f*WINX, red, 1.0f, 0.3f);
   lens_flare[4].init("textures/flare1.bmp", 1.0f, 0.04f*WINX, red, 1.0f, 0.3f);
   lens_flare[5].init("textures/flare6.bmp", 1.25f, 0.07f*WINX, red, 1.0f, 0.5f);
   lens_flare[6].init("textures/flare6.bmp", 1.4f, 0.02f*WINX, red, 1.0f, 0.6f);
   lens_flare[7].init("textures/flare6.bmp", 1.6f, 0.04f*WINX, red, 1.0f, 0.4f);
   lens_flare[8].init("textures/flare6.bmp", 2.0f, 0.03f*WINX, violet, 1.0f, 1.0f);
   lens_glare.init("textures/flare3.bmp", 0.0f, 0.33f*WINX, white, 1.0f, 0.25f);
*/

   vector4uc yellow = { 255, 255, 0, 0 };
   lens_flare[0].init("textures/flare3.bmp", 0.2f, 0.04f*WINX, yellow, 1.0f, 0.6f);
   lens_flare[1].init("textures/flare4.bmp", 0.4f, 0.1f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[2].init("textures/flare2.bmp", 0.6f, 0.2f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[3].init("textures/flare5.bmp", 0.8f, 0.05f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[4].init("textures/flare1.bmp", 1.0f, 0.04f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[5].init("textures/flare6.bmp", 1.25f, 0.07f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[6].init("textures/flare6.bmp", 1.4f, 0.02f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[7].init("textures/flare6.bmp", 1.6f, 0.04f*WINX, yellow, 1.0f, 1.0f);
   lens_flare[8].init("textures/flare6.bmp", 2.0f, 0.03f*WINX, yellow, 1.0f, 1.0f);
   lens_glare.init("textures/flare3.bmp", 0.0f, 0.33f*WINX, white, 1.0f, 1.0f);
}


/* **************************************************************
************************************************************** */
void vfx_dazzle_the_crowd(void *buffer) {

   gameatom *atr;
   quark    *sol, *sol_parent;

   vfx_timer += complex->timer.speedscale;
   if (vfx_timer > 256)
      vfx_timer -= 256;

   for (sol=NULL, atr=pixcon_object_list; atr; atr=(gameatom *)atr->next) {
      sol = atr->find(NULL, OBJECT_PHOTON, NULL, &sol_parent);
      if (sol)
         break;
   }
   
   if (sol && (sol_parent->old_state.state_flags & STATE_MASK_BOUND) == STATE_FLAG_BSPHERE)
      vfx_lens_flare(buffer, sol_parent, atr);
}
